////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.weightpoints;

/***********************************************************************************************
Provides calculations for weight points
***********************************************************************************************/
public class WeightPointsHelper {

  /***********************************************************************************************
  Calculates the points plus value for alcoholic foods, using the <i>protein</i>, <i>carbohydrates</i>, <i>fat</i>, <i>fiber</i>, <i>alcohol</i> and <i>sugarAlcohol</i>
  @param protein the amount of protein
  @param carbohydrates the amount of carbohydrates
  @param fat the amount of fat
  @param fiber the amount of fiber
  @param alcohol the amount of alcohol
  @param sugarAlcohol the amount of sugar alcohol
  @param halfPoints <b>true</b> if half-points are allowed<br />
                    <b>false</b> if only full point values are allowed
  @return the points value 
  ***********************************************************************************************/
  public static double calculateAlcoholPointsPlus(double protein,double carbohydrates,double fat,double fiber,double alcohol,double sugarAlcohol,boolean halfPoints){
    double result = ((double)((int)((protein / 10.9375 + carbohydrates / 9.2105 + fat / 3.8889 - fiber / 12.5 + alcohol / 3.0147 - sugarAlcohol / 23.0263) * 10))) / 10;
    return roundResult(result,halfPoints);
  }


  /***********************************************************************************************
  Calculates the Australian points value for the <i>kilojoules</i> and <i>saturatedFat</i>
  @param kilojoules the number of kilojoules
  @param saturatedFat the grams of saturated fat
  @param halfPoints <b>true</b> if half-points are allowed<br />
                    <b>false</b> if only full point values are allowed
  @return the points value 
  ***********************************************************************************************/
  public static double calculateAustraliaPoints(double kilojoules,double saturatedFat,boolean halfPoints){
    double result = ((double)((int)((kilojoules / 290 + saturatedFat / 4.05d) * 10))) / 10;
    return roundResult(result,halfPoints);
  }


  /***********************************************************************************************
  Calculates the points plus value for the <i>protein</i>, <i>carbohydrates</i>, <i>fat</i> and <i>fiber</i>
  @param protein the amount of protein
  @param carbohydrates the amount of carbohydrates
  @param fat the amount of fat
  @param fiber the amount of fiber
  @param halfPoints <b>true</b> if half-points are allowed<br />
                    <b>false</b> if only full point values are allowed
  @return the points value 
  ***********************************************************************************************/
  public static double calculatePointsPlus(double protein,double carbohydrates,double fat,double fiber,boolean halfPoints){
    double result = ((double)((int)((protein / 10.9375 + carbohydrates / 9.2105 + fat / 3.8889 - fiber / 12.5) * 10))) / 10;
    return roundResult(result,halfPoints);
  }


  /***********************************************************************************************
  Calculates the pro points value for the <i>protein</i>, <i>carbohydrates</i>, <i>fat</i> and <i>fiber</i>
  @param protein the amount of protein
  @param carbohydrates the amount of carbohydrates
  @param fat the amount of fat
  @param fiber the amount of fiber
  @param halfPoints <b>true</b> if half-points are allowed<br />
                    <b>false</b> if only full point values are allowed
  @return the points value 
  ***********************************************************************************************/
  public static double calculateProPoints(double protein,double carbohydrates,double fat,double fiber,boolean halfPoints){
    double result = ((double)((int)((protein / 10.9375 + carbohydrates / 9.2105 + fat / 3.8889 + fiber / 35) * 10))) / 10;
    return roundResult(result,halfPoints);
  }


  /***********************************************************************************************
  Calculates the UK points value for the <i>calories</i> and <i>saturatedFat</i>
  @param calories the number of calories
  @param saturatedFat the grams of saturated fat
  @param halfPoints <b>true</b> if half-points are allowed<br />
                    <b>false</b> if only full point values are allowed
  @return the points value 
  ***********************************************************************************************/
  public static double calculateUKPoints(double calories,double saturatedFat,boolean halfPoints){
    double result = ((double)((int)((calories / 70 + saturatedFat / 4.05d) * 10))) / 10;
    return roundResult(result,halfPoints);
  }


  /***********************************************************************************************
  Calculates the USA points value for the <i>calories</i>, <i>fat</i> and <i>fiber</i>
  @param calories the number of calories
  @param fat the grams of total fat
  @param fiber the grams of fiber
  @param halfPoints <b>true</b> if half-points are allowed<br />
                    <b>false</b> if only full point values are allowed
  @return the points value 
  ***********************************************************************************************/
  public static double calculateUSAPoints(double calories,double fat,double fiber,boolean halfPoints){
    double result = ((double)((int)((calories / 50 + fat / 12 - Math.min(fiber,4) / 5) * 10))) / 10;
    return roundResult(result,halfPoints);
  }


  /***********************************************************************************************
  Rounds the result according to whether the user wants to use decimal points or not
  @param result the points value to round
  @param halfPoints <b>true</b> if half-points are allowed<br />
                    <b>false</b> if only full point values are allowed
  @return the rounded points value 
  ***********************************************************************************************/
  public static double roundResult(double result,boolean halfPoints){
    int fullValue = ((int)(result * 10)) / 10;
    int remainder = (int)(((double)result - (double)fullValue) * 10);

    //SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(this);
    //boolean decimalPoints = settings.getBoolean("DecimalPoints",true);
    //boolean zeroPoints = settings.getBoolean("ZeroPoints",false);
    boolean zeroPoints = true;

    if (halfPoints) {
      if (remainder < 3) {
        remainder = 0;
      }
      else if (remainder < 8) {
        remainder = 5;
      }
      else if (remainder < 10) {
        remainder = 0;
        fullValue += 1;
      }

      if (fullValue == 0 && !zeroPoints) {
        remainder = 5;
      }

      result = Double.parseDouble(fullValue + "." + remainder);
    }
    else {
      if (remainder < 5) {
        remainder = 0;
      }
      else {
        remainder = 0;
        fullValue += 1;
      }

      if (fullValue == 0 && !zeroPoints) {
        fullValue = 1;
      }

      result = fullValue;
    }

    return result;
  }

}